﻿using FLY.Thick.Base.UI;
using FLY.Thick.BlowingScan.Client;
using FLY.Thick.BlowingScan.Common;
using FLY.Thick.BlowingScan.IService;
using FObjBase;
using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using Unity;

namespace FLY.Thick.BlowingScan.UI.Client
{
    /// <summary>
    /// Page_ProfileBlowing.xaml 的交互逻辑
    /// </summary>
    public partial class PgProfileBlowing : Page
    {
        PgProfileBlowingVm viewModel;
        public PgProfileBlowing()
        {
            InitializeComponent();
        }
        [InjectionMethod]
        public void Init(IBlowingProfileService profileService)
        {
            viewModel = new PgProfileBlowingVm();
            viewModel.Init(profileService, listview_profile);
            this.DataContext = viewModel;
        }
    }
    public class PgProfileBlowingVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        #region 参数
        public ObservableCollection<string> ProfileList { get; } = new ObservableCollection<string>();
        public BlowingProfileParam Param { get; } = new BlowingProfileParam();
        #endregion

        #region Command
        public RelayCommand ReadCmd { get; }
        public RelayCommand ApplyCmd { get; }
        public RelayCommand DelCmd { get; }

        public RelayCommand AhelperCmd { get; }

        #endregion

        IBlowingProfileService profileService;
        ListBox listview_profile;

        public PgProfileBlowingVm()
        {
            ReadCmd = new RelayCommand(Read);
            ApplyCmd = new RelayCommand(Apply);
            DelCmd = new RelayCommand(Del);
            AhelperCmd = new RelayCommand(Ahelper);
        }

        public void Init(IBlowingProfileService profileService, ListBox listview_profile)
        {
            this.profileService = profileService;
            this.listview_profile = listview_profile;

            Param.Copy(this.profileService.Param);

            this.profileService.GetList(
                new AsyncCBHandler(
                    delegate (object AsyncState, object retData)
                    {
                        List<string> list = (List<string>)retData;
                        ProfileList.Clear();
                        for (int i = 0; i < list.Count(); i++)
                            ProfileList.Add(list[i]);

                        if (ProfileList.Contains(Param.PName))
                        {
                            this.listview_profile.SelectedItem = ProfileList.First(s => s == Param.PName);
                        }
                    }), null);
            this.profileService.Param.PropertyChanged += Param_PropertyChanged;
        }

        private void Param_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //TODO
        }


        private void Read()
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                profileService.Read(
                    productname,
                    new AsyncCBHandler(
                    delegate (object AsyncState, object retData)
                    {
                        var p = (BlowingProfileParam)retData;
                        Misc.PropertiesManager.CopyTo(p, Param);
                    }), null);
            }
        }

        private void Apply()
        {
            if (Param.Target <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "目标值<=0 异常");
                return;
            }

            if (Param.TolerancePercent <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "公差 % <=0 异常");
                return;
            }

            Misc.PropertiesManager.CopyTo(Param, profileService.Param);
            profileService.Apply();

            if (!ProfileList.Contains(Param.PName))
            {
                ProfileList.Add(Param.PName);
            }
            FLY.ControlLibrary.Window_Tip.Show("应用 & 保存成功",
                Param.PName,
                TimeSpan.FromSeconds(2));

        }

        private void Del()
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                profileService.Del(productname);
                ProfileList.Remove(productname);
                FLY.ControlLibrary.Window_Tip.Show("删除成功",
                    productname,
                    TimeSpan.FromSeconds(2));
            }
        }

        private void Ahelper()
        {

            WdAHelper w = new WdAHelper();

            w.Init(Param.K);
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(listview_profile);
            if (w.ShowDialog() == true)
            {
                Param.K = w.A;
            }
        }
    }
}
